@@ -58,6 +58,7 @@ end
         @time @safetestset "Table Inference Tests" include("downstream/tables.jl")
         @time @safetestset "Default linsolve with structure" include("downstream/default_linsolve_structure.jl")
         @time @safetestset "Callback Merging Tests" include("downstream/callback_merging.jl")
+        @time @safetestset "Callback Detection Tests" include("downstream/callback_detection.jl")
         @time @safetestset "LabelledArrays Tests" include("downstream/labelledarrays.jl")
         @time @safetestset "GTPSA Tests" include("downstream/gtpsa.jl")
         @time @safetestset "SubArray Support" include("downstream/subarray_support.jl")
@@ -78,6 +79,7 @@ end
             @time @safetestset "Ensemble AD Tests" include("downstream/ensemble_ad.jl")
         @time @safetestset "Community Callback Tests" include("downstream/community_callback_tests.jl")
         @time @safetestset "AD via ode with complex numbers" include("downstream/complex_number_ad.jl")
+        @time @safetestset "Callback AD Tests" include("downstream/callback_ad.jl")
         @time @testset "Distributed Ensemble Tests" include("downstream/distributed_ensemble.jl")
     end
 
